// FitObjectToHeadController.js
// Version: 0.0.1
// Event: Lens Initialized
// Description: Pins and slides reference points to Face Mesh


//@input float verticalOffset = 0.0 {"widget": "slider", "min": -30.0, "max": 30.0, "step": 0.1}
//@input float width = 13.0 {"widget": "slider", "min": 0, "max": 30.0, "step": 0.1}
//@input float depth = 19.0 {"widget": "slider", "min": 0, "max": 30.0, "step": 0.1}
//@input SceneObject hat
//@input Component.BaseMeshVisual mesh


// Touch indicators
var touching = false;
var touchVec = null;

// Limits
var xcv = 0.1;
var ycv = 0.035;

// Reference points
var leftRefPointOffset = vec2.zero();
var backRefPointOffset = vec2.zero();
var frontRefPointOffset = vec2.zero();

var leftRefPoint;
var rightRefPoint;
var frontRefPoint;
var backRefPoint;
var leftRefPointTrs;
var rightRefPointTrs;
var frontRefPointTrs;
var backRefPointTrs;

// Hat transform
var hatTrs = script.hat.getTransform();

// Default positions
var leftPointDef = new vec2(0.28, 0.77);
var rightPointDef = new vec2(0.72, 0.77);
var backPointDef = new vec2(0.95, 0.82);
var frontPointDef = new vec2(0.48, 0.77);

var lateUpdateEvent;


function onTouchStarted(data) {
    touching = true;
    touchVec = data.getTouchPosition();
}


function onTouchMoved(data) {
    if (!touching) {
        return;
    }

    // Calculate offsets
    var pos = data.getTouchPosition();
    var dir = pos.sub(touchVec);
    dir = new vec2(Math.min(xcv, Math.max(-xcv, dir.x)), Math.min(ycv, Math.max(-ycv, dir.y)));

    leftRefPointOffset = new vec2(0, dir.x * 0.5);
    frontRefPointOffset = new vec2(0, -dir.y);
    backRefPointOffset = new vec2(-dir.y * 2, dir.y * 2);

    updateHat(leftRefPointOffset, frontRefPointOffset, backRefPointOffset);
}


function onTouchEnded(data) {
    touching = false;
}


function onLateUpdate() {
    // Head position is available on late update
    // Once we have head, we can apply initial adjustment from Inspector panel
    // Then we disable the late update event.
    var zero = vec2.zero();
    updateHat(zero, zero, zero);
    lateUpdateEvent.enabled = false;
}


function updateHat(leftRefPointOffset, frontRefPointOffset, backRefPointOffset) {
    // Apply offset
    leftRefPoint.pinUV = leftPointDef.add(leftRefPointOffset);
    rightRefPoint.pinUV = rightPointDef.sub(leftRefPointOffset);
    frontRefPoint.pinUV = frontPointDef.add(frontRefPointOffset);
    backRefPoint.pinUV = backPointDef.add(backRefPointOffset);

    // Get marker positions
    var leftRefPointPos = leftRefPointTrs.getWorldPosition();
    var rightRefPointPos = rightRefPointTrs.getWorldPosition();
    var frontRefPointPos = frontRefPointTrs.getWorldPosition();
    var backRefPointPos = backRefPointTrs.getWorldPosition();

    // Calculate centroid position
    var sumVec = leftRefPointPos;
    sumVec = sumVec.add(rightRefPointPos);
    sumVec = sumVec.add(frontRefPointPos);
    sumVec = sumVec.add(backRefPointPos);
    var centroidPos = sumVec.uniformScale(0.25);

    // Set hat to centroid
    centroidPos = centroidPos.add(new vec3(0, script.verticalOffset, 0));
    hatTrs.setWorldPosition(centroidPos);
    var hatRotation = quat.fromEulerAngles(-frontRefPointOffset.y * 4, 0, -leftRefPointOffset.y * 4);
    hatTrs.setLocalRotation(hatRotation);

    // Calculate distances
    var xDist = leftRefPointPos.sub(rightRefPointPos).length;
    var zDist = backRefPointPos.sub(frontRefPointPos).length;

    // Set hat scale
    var hatScale = new vec3(script.width / xDist, script.width / xDist, script.depth / zDist);
    hatTrs.setWorldScale(hatScale);
}

function makePoint(name, defaultUV) {
    var so = global.scene.createSceneObject(name);
    var comp = so.createComponent("Component.PinToMeshComponent");
    comp.target = script.mesh;
    comp.preferredUVLayerIndex = 1;
    comp.pinUV = defaultUV;
    return comp;
}


function init() {
    // Create refrence points
    var parent = script.getSceneObject();
    leftRefPoint = makePoint("leftRefPoint", leftPointDef);
    leftRefPointTrs = leftRefPoint.getSceneObject().getTransform();
    leftRefPointTrs.getSceneObject().setParent(parent);

    rightRefPoint = makePoint("rightRefPoint", rightPointDef);
    rightRefPointTrs = rightRefPoint.getSceneObject().getTransform();
    rightRefPointTrs.getSceneObject().setParent(parent);

    backRefPoint = makePoint("backRefPoint", backPointDef);
    backRefPointTrs = backRefPoint.getSceneObject().getTransform();
    backRefPointTrs.getSceneObject().setParent(parent);

    frontRefPoint = makePoint("frontRefPoint", frontPointDef);
    frontRefPointTrs = frontRefPoint.getSceneObject().getTransform();
    frontRefPointTrs.getSceneObject().setParent(parent);

    // Create events
    script.createEvent("TouchStartEvent").bind(onTouchStarted);
    script.createEvent("TouchMoveEvent").bind(onTouchMoved);
    script.createEvent("TouchEndEvent").bind(onTouchEnded);

    lateUpdateEvent = script.createEvent("LateUpdateEvent");
    lateUpdateEvent.bind(onLateUpdate);

    // Block default screen touching
    global.touchSystem.touchBlocking = true;
    global.touchSystem.enableTouchBlockingException("TouchTypeDoubleTap", true);
}


init();
